@@ -13,7 +13,8 @@ int db_add_process(unsigned int *id, unsigned int parent_id,
 int db_add_exit(unsigned int id, int exitcode);
 int db_add_first_process(unsigned int *id, const char *working_dir);
 int db_add_file_open(unsigned int process,
-                     const char *name, unsigned int mode);
+                     const char *name, unsigned int mode,
+                     int is_dir);
 int db_add_exec(unsigned int process, const char *binary,
                 const char *const *argv, const char *const *envp,
                 const char *workingdir);
