@@ -6,9 +6,23 @@
 #   reprozip/reprozip/common.py
 #   reprounzip/reprounzip/common.py
 
+"""Common functions between reprozip and reprounzip.
+
+This module contains functions that are specific to the reprozip software and
+its data formats, but that are shared between the reprozip and reprounzip
+packages. Because the packages can be installed separately, these functions are
+in a separate module which is duplicated between the packages.
+
+As long as these are small in number, they are not worth putting in a separate
+package that reprozip and reprounzip would both depend on.
+
+"""
+
 from __future__ import unicode_literals
 
 from datetime import datetime
+from distutils.version import LooseVersion
+import logging
 from rpaths import PosixPath
 import yaml
 
@@ -39,10 +53,10 @@ class File(CommonEqualityMixin):
 
 
 class Package(CommonEqualityMixin):
-    def __init__(self, name, version, files=[], packfiles=True, size=None):
+    def __init__(self, name, version, files=None, packfiles=True, size=None):
         self.name = name
         self.version = version
-        self.files = list(files)
+        self.files = list(files) if files is not None else []
         self.packfiles = packfiles
         self.size = size
 
@@ -60,10 +74,14 @@ class InvalidConfig(ValueError):
 
 
 def read_files(files, File=File):
+    if files is None:
+        return []
     return [File(PosixPath(f)) for f in files]
 
 
 def read_packages(packages, File=File, Package=Package):
+    if packages is None:
+        return []
     new_pkgs = []
     for pkg in packages:
         pkg['files'] = read_files(pkg['files'], File)
@@ -71,24 +89,42 @@ def read_packages(packages, File=File, Package=Package):
     return new_pkgs
 
 
-def load_config(filename, File=File, Package=Package):
+def load_config(filename, canonical, File=File, Package=Package):
     with filename.open(encoding='utf-8') as fp:
         config = yaml.safe_load(fp)
 
-    keys_ = set(config.keys())
+    ver = LooseVersion(config['version'])
+
+    keys_ = set(config)
     if 'version' not in keys_:
         raise InvalidConfig("Missing version")
-    elif config['version'] not in ('0.2', '0.2.1'):
+    # Accepts versions from 0.2 to 0.4 inclusive
+    elif not LooseVersion('0.2') <= ver < LooseVersion('0.5'):
         raise InvalidConfig("Unknown version")
     elif not keys_.issubset(set(['version', 'runs',
-                                 'packages', 'other_files'])):
+                                 'packages', 'other_files',
+                                 'additional_patterns'])):
         raise InvalidConfig("Unrecognized sections")
 
     runs = config.get('runs', [])
     packages = read_packages(config.get('packages', []), File, Package)
     other_files = read_files(config.get('other_files', []), File)
 
-    return runs, packages, other_files
+    # Adds 'input_files' and 'output_files' keys to runs
+    for run in runs:
+        if 'input_files' not in run:
+            run['input_files'] = {}
+        if 'output_files' not in run:
+            run['output_files'] = {}
+
+    if canonical:
+        if 'additional_patterns' in config:
+            raise InvalidConfig("Canonical configuration file shouldn't have "
+                                "additional_patterns key anymore")
+        return runs, packages, other_files
+    else:
+        additional_patterns = config.get('additional_patterns') or []
+        return runs, packages, other_files, additional_patterns
 
 
 def write_file(fp, fi, indent=0):
@@ -115,11 +151,12 @@ def write_package(fp, pkg, indent=0):
     if pkg.size is not None:
         fp.write("%s      # Installed package size: %s\n" % (
                  indent_str, hsize(pkg.size)))
-    for fi in pkg.files:
+    for fi in sorted(pkg.files, key=lambda fi_: fi_.path):
         write_file(fp, fi, indent + 1)
 
 
-def save_config(filename, runs, packages, other_files, reprozip_version):
+def save_config(filename, runs, packages, other_files, reprozip_version,
+                canonical=False):
     dump = lambda x: yaml.safe_dump(x, encoding='utf-8', allow_unicode=True)
     with filename.open('w', encoding='utf-8', newline='\n') as fp:
         # Writes preamble
@@ -127,17 +164,19 @@ def save_config(filename, runs, packages, other_files, reprozip_version):
 # ReproZip configuration file
 # This file was generated by reprozip {version} at {date}
 
-# You might want to edit this file before running the packer
-# See 'reprozip pack -h' for help
+{what}
 
 # Run info
 version: "{version!s}"
 """.format(version=escape(reprozip_version),
-           date=datetime.now().isoformat()))
+           date=datetime.now().isoformat(),
+           what="# It was generated by the packer and you shouldn't need to "
+           "edit it" if canonical
+           else "# You might want to edit this file before running the "
+           "packer\n# See 'reprozip pack -h' for help"))
         fp.write(dump({'runs': runs}).decode('utf-8'))
         fp.write("""\
 
-
 # Files to pack
 # All the files below were used by the program; they will be included in the
 # generated package
@@ -149,7 +188,7 @@ packages:
 """)
 
         # Writes files
-        for pkg in sorted(packages, key=lambda pkg: pkg.name):
+        for pkg in sorted(packages, key=lambda p: p.name):
             write_package(fp, pkg)
 
         fp.write("""\
@@ -160,3 +199,42 @@ other_files:
 """)
         for f in sorted(other_files, key=lambda fi: fi.path):
             write_file(fp, f)
+
+        if not canonical:
+            fp.write("""\
+
+# If you want to include additional files in the pack, you can list additional
+# patterns of files that will be included
+# Example:
+#additional_patterns:
+#  - /etc/apache2/**  # Everything under apache2/
+#  - /var/log/apache2/*.log  # Log files directly under apache2/
+#  - /var/lib/lxc/*/rootfs/home/**/*.py  # All Python files of all users in
+#    # that container
+""")
+
+
+class LoggingDateFormatter(logging.Formatter):
+    converter = datetime.fromtimestamp
+
+    def formatTime(self, record, datefmt=None):
+        ct = self.converter(record.created)
+        t = ct.strftime("%H:%M:%S")
+        s = "%s.%03d" % (t, record.msecs)
+        return s
+
+
+def setup_logging(tag, verbosity):
+    levels = [logging.CRITICAL, logging.WARNING, logging.INFO, logging.DEBUG]
+    level = levels[min(verbosity, 3)]
+
+    fmt = "[%s] %%(asctime)s %%(levelname)s: %%(message)s" % tag
+
+    formatter = LoggingDateFormatter(fmt)
+
+    handler = logging.StreamHandler()
+    handler.setFormatter(formatter)
+
+    logger = logging.getLogger()
+    logger.setLevel(level)
+    logger.addHandler(handler)
