@@ -2,6 +2,14 @@
 # This file is part of ReproZip which is released under the Revised BSD License
 # See file LICENSE for full license details.
 
+"""Entry point for the reprozip utility.
+
+This contains :func:`~reprozip.main.main`, which is the entry point declared to
+setuptools. It is also callable directly.
+
+It dispatchs to other routines, or handles the testrun command.
+"""
+
 from __future__ import unicode_literals
 
 import argparse
@@ -13,7 +21,9 @@ from rpaths import Path
 import sqlite3
 import sys
 
+from reprozip import __version__ as reprozip_version
 from reprozip import _pytracer
+from reprozip.common import setup_logging
 import reprozip.pack
 import reprozip.tracer.trace
 from reprozip.utils import PY3
@@ -124,6 +134,8 @@ def testrun(args):
             argv = [args.arg0] + args.cmdline[1:]
         else:
             argv = args.cmdline
+        logging.debug("Starting tracer, binary=%r, argv=%r" % (
+                      args.cmdline[0], argv))
         c = _pytracer.execute(args.cmdline[0], argv, database.path,
                               args.verbosity)
         print("\n\n-----------------------------------------------------------"
@@ -193,10 +205,21 @@ def main():
         sys.stdout = writer(sys.stdout)
         sys.stderr = writer(sys.stderr)
 
+    # http://bugs.python.org/issue13676
+    # This prevents reprozip from reading argv and envp arrays from trace
+    if sys.version_info < (2, 7, 3):
+        sys.stderr.write("Error: your version of Python, %s, is not "
+                         "supported\nVersions before 2.7.3 are affected by "
+                         "bug 13676 and will not work with ReproZip\n" %
+                         sys.version.split(' ', 1)[0])
+        sys.exit(1)
+
     # Parses command-line
 
     # General options
     options = argparse.ArgumentParser(add_help=False)
+    options.add_argument('--version', action='version',
+                         version="reprozip version %s" % reprozip_version)
     options.add_argument('-v', '--verbose', action='count', default=1,
                          dest='verbosity',
                          help="augments verbosity level")
@@ -209,7 +232,8 @@ def main():
             help="do not try identify which package each file comes from")
 
     parser = argparse.ArgumentParser(
-            description="Reproducible experiments tool.",
+            description="reprozip is the ReproZip component responsible for "
+                        "tracing and packing the execution of an experiment",
             epilog="Please report issues to reprozip-users@vgc.poly.edu",
             parents=[options])
     subparsers = parser.add_subparsers(title="commands", metavar='')
@@ -256,8 +280,7 @@ def main():
     parser_pack.set_defaults(func=pack)
 
     args = parser.parse_args()
-    levels = [logging.CRITICAL, logging.WARNING, logging.INFO, logging.DEBUG]
-    logging.basicConfig(level=levels[min(args.verbosity, 3)])
+    setup_logging('REPROZIP', args.verbosity)
     if 'cmdline' in args and not args.cmdline:
         parser.error("missing command-line")
     args.func(args)
