@@ -2,45 +2,66 @@
 # This file is part of ReproZip which is released under the Revised BSD License
 # See file LICENSE for full license details.
 
+"""Packing logic for reprozip.
+
+This module contains the :func:`~reprozip.pack.pack` function and associated
+utilities that are used to build the .rpz pack file from the trace SQLite file
+and config YAML.
+"""
+
 from __future__ import unicode_literals
 
+import itertools
 import logging
 import os
 from rpaths import Path
-import sqlite3
 import sys
 import tarfile
 
 from reprozip import __version__ as reprozip_version
-from reprozip.common import FILE_WRITE, FILE_WDIR, load_config, save_config
-from reprozip.tracer.canonicalize import canonicalize_config
-from reprozip.utils import PY3
+from reprozip.common import File, load_config, save_config
+from reprozip.tracer.linux_pkgs import identify_packages
+from reprozip.tracer.trace import merge_files
 
 
-def list_directories(database):
-    if PY3:
-        # On Python 3, connect() only accepts unicode
-        conn = sqlite3.connect(str(database))
+def expand_patterns(patterns):
+    files = set()
+    dirs = set()
+
+    # Finds all matching paths
+    for pattern in patterns:
+        if logging.root.isEnabledFor(logging.DEBUG):
+            logging.debug("Expanding pattern %r into %d paths" % (
+                          pattern,
+                          len(list(Path('/').recursedir(pattern)))))
+        for path in Path('/').recursedir(pattern):
+            if path.is_dir():
+                dirs.add(path)
+            else:
+                files.add(path)
+
+    # Don't include directories whose files are included
+    non_empty_dirs = set([Path('/')])
+    for p in files | dirs:
+        path = Path('/')
+        for c in p.components[1:]:
+            path = path / c
+            non_empty_dirs.add(path)
+
+    # Builds the final list
+    return [File(p) for p in itertools.chain(dirs - non_empty_dirs, files)]
+
+
+def canonicalize_config(runs, packages, other_files, additional_patterns,
+                        sort_packages):
+    add_files = expand_patterns(additional_patterns)
+    if sort_packages:
+        add_files, add_packages = identify_packages(add_files)
     else:
-        conn = sqlite3.connect(database.path)
-    conn.row_factory = sqlite3.Row
-    cur = conn.cursor()
-    executed_files = cur.execute(
-            '''
-            SELECT name, mode
-            FROM opened_files
-            WHERE mode = ? OR mode = ?
-            ''',
-            (FILE_WDIR, FILE_WRITE))
-    executed_files = ((Path(n), m) for n, m in executed_files)
-    # If WDIR, the name is a folder that was used as working directory
-    # If WRITE, the name is a file that was written to; its directory must
-    # exist
-    result = set(n if m == FILE_WDIR else n.parent
-                 for n, m in executed_files)
-    cur.close()
-    conn.close()
-    return result
+        add_packages = []
+    other_files, packages = merge_files(add_files, add_packages,
+                                        other_files, packages)
+    return runs, packages, other_files
 
 
 def data_path(filename, prefix=Path('DATA')):
@@ -92,16 +113,16 @@ def pack(target, directory, sort_packages):
     """
     if target.exists():
         # Don't overwrite packs...
-        sys.stderr.write("Error: Target file exists!\n")
+        logging.critical("Target file exists!")
         sys.exit(1)
 
     # Reads configuration
     configfile = directory / 'config.yml'
     if not configfile.is_file():
-        sys.stderr.write("Error: Configuration file does not exist!\n"
+        logging.critical("Configuration file does not exist!\n"
                          "Did you forget to run 'reprozip trace'?\n"
                          "If not, you might want to use --dir to specify an "
-                         "alternate location.\n")
+                         "alternate location.")
         sys.exit(1)
     runs, packages, other_files, additional_patterns = load_config(
             configfile,
@@ -137,22 +158,15 @@ def pack(target, directory, sort_packages):
 
     # Add the rest of the files
     logging.info("Adding other files...")
-    files = []
+    files = set()
     for f in other_files:
         if not Path(f.path).exists():
             logging.warning("Missing file %s" % f.path)
         else:
             tar.add_data(f.path)
-            files.append(f)
+            files.add(f)
     other_files = files
 
-    # Makes sure all the directories used as working directories are packed
-    # (they already do if files from them are used, but empty directories do
-    # not get packed inside a tar archive)
-    for directory in list_directories(trace):
-        if directory.is_dir():
-            tar.add_data(directory)
-
     logging.info("Adding metadata...")
     # Stores pack version
     fd, manifest = Path.tempfile(prefix='reprozip_', suffix='.txt')
