@@ -2,6 +2,16 @@
 # This file is part of ReproZip which is released under the Revised BSD License
 # See file LICENSE for full license details.
 
+"""Package identification routines.
+
+This module contains the :func:`~reprozip.tracer.linux_pkgs.identify_packages`
+function that sorts a list of files between their distribution packages,
+depending on what Linux distribution we are running on.
+
+Currently supported package managers:
+- dpkg (Debian, Ubuntu)
+"""
+
 from __future__ import unicode_literals
 
 import platform
@@ -9,6 +19,7 @@ from rpaths import Path
 import subprocess
 
 from reprozip.common import Package
+from reprozip.utils import listvalues
 
 
 magic_dirs = ('/dev', '/proc', '/sys')
@@ -17,7 +28,7 @@ system_dirs = ('/bin', '/etc', '/lib', '/sbin', '/usr', '/var')
 
 class DpkgManager(object):
     def __init__(self):
-        self.unknown_files = []
+        self.unknown_files = set()
         self.packages = {}
         self.package_files = {}
 
@@ -29,7 +40,7 @@ class DpkgManager(object):
         # If it's not in a system directory, no need to look for it
         if (f.path.lies_under('/usr/local') or
                 not any(f.path.lies_under(c) for c in system_dirs)):
-            self.unknown_files.append(f)
+            self.unknown_files.add(f)
             return
 
         # Looks in our cache
@@ -41,7 +52,7 @@ class DpkgManager(object):
 
         # Stores the file
         if pkgname is None:
-            self.unknown_files.append(f)
+            self.unknown_files.add(f)
         else:
             if pkgname in self.packages:
                 self.packages[pkgname].add_file(f)
@@ -50,21 +61,20 @@ class DpkgManager(object):
 
     def _get_package_for_file(self, filename):
         p = subprocess.Popen(['dpkg', '-S', filename.path],
-                             stdout=subprocess.PIPE)
-        try:
-            for l in p.stdout:
-                pkgname, f = l.split(b': ', 1)
-                f = Path(f.strip())
-                # 8-bit safe encoding, because this might be a localized error
-                # message (that we don't care about)
-                pkgname = (pkgname.decode('iso-8859-1')
-                                  .split(':', 1)[0])    # Removes :arch
-                self.package_files[f] = pkgname
-                if f == filename:
-                    if ' ' not in pkgname:
-                        return pkgname
-        finally:
-            p.wait()
+                             stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE)
+        out, err = p.communicate()
+        for l in out.splitlines():
+            pkgname, f = l.split(b': ', 1)
+            f = Path(f.strip())
+            # 8-bit safe encoding, because this might be a localized error
+            # message (that we don't care about)
+            pkgname = (pkgname.decode('iso-8859-1')
+                              .split(':', 1)[0])    # Removes :arch
+            self.package_files[f] = pkgname
+            if f == filename:
+                if ' ' not in pkgname:
+                    return pkgname
         return None
 
     def _create_package(self, pkgname, files):
@@ -107,4 +117,4 @@ def identify_packages(files):
     for f in files:
         manager.search_for_file(f)
 
-    return manager.unknown_files, manager.packages.values()
+    return manager.unknown_files, listvalues(manager.packages)
